# -*- coding: utf-8 -*-

# The following code is a reworked version of the one available at
# http://nbviewer.ipython.org/gist/BenLangmead/6665861


class SuffixTree(object):
    
    rootchar= "."
    
    class Node(object):
        def __init__(self, lab):
            self.label = lab # label on path leading to this node
            self.children = {}  # outgoing edges; maps characters to nodes
    
    def __init__(self, s):
        """ Make suffix tree, without suffix links, from s in quadratic time
            and linear space """
        s += '$'
        self.root = self.Node(SuffixTree.rootchar)
        self.root.children[s[0]] = self.Node(s) # trie for just longest suf
        # add the rest of the suffixes, from longest to shortest
        for i in xrange(1, len(s)):
            # start at root; we’ll walk down as far as we can go
            cur = self.root
            j = i
            while j < len(s):
                if s[j] in cur.children:
                    child = cur.children[s[j]]
                    lab = child.label
                    # Walk along edge until we exhaust edge label or
                    # until we mismatch
                    k = j+1 
                    while k-j < len(lab) and s[k] == lab[k-j]:
                        k += 1
                    if k-j == len(lab):
                        cur = child # we exhausted the edge
                        j = k
                    else:
                        # we fell off in middle of edge
                        cExist, cNew = lab[k-j], s[k]
                        # create “mid”: new node bisecting edge
                        mid = self.Node(lab[:k-j])
                        mid.children[cNew] = self.Node(s[k:])
                        # original child becomes mid’s child
                        mid.children[cExist] = child
                        # original child’s label is curtailed
                        child.label = lab[k-j:]
                        # mid becomes new child of original parent
                        cur.children[s[j]] = mid
                else:
                    # Fell off tree at a node: make new edge hanging off it
                    cur.children[s[j]] = self.Node(s[j:])
    
    def followPath(self, s):
        """ Follow path given by s.  If we fall off tree, return None.  If we
            finish mid-edge, return (node, offset) where 'node' is child and
            'offset' is label offset.  If we finish on a node, return (node,
            None). """
        cur = self.root
        i = 0
        while i < len(s):
            c = s[i]
            if c not in cur.children:
                return (None, None) # fell off at a node
            child = cur.children[s[i]]
            lab = child.label
            j = i+1
            while j-i < len(lab) and j < len(s) and s[j] == lab[j-i]:
                j += 1
            if j-i == len(lab):
                cur = child # exhausted edge
                i = j
            elif j == len(s):
                return (child, j-i) # exhausted query string in middle of edge
            else:
                return (None, None) # fell off in the middle of the edge
        return (cur, None) # exhausted query string at internal node
    

# Added part
 
    def newick(self):
        
        def in_newick(node):
            sout=''
            if node.children:
                sout += '('
                node_keys = sorted(node.children.keys())
                lk = len(node_keys)
                for i,k in enumerate(node_keys):
                    sout += in_newick(node.children[k])
                    if i+1 < lk:
                        sout += ','
                sout += ')'
            if node.label:
                sout += node.label
            return sout
        
        
        return in_newick(self.root)
            
